@@ -1,7 +1,7 @@
 // synthv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 #include "synthv1_param.h"
 
 #include "synthv1_wave.h"
+#include "synthv1_sched.h"
 
 #include "synthv1widget_config.h"
 
@@ -61,6 +62,9 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.setupUi(this);
 
+	// Init sched notifier.
+	m_sched_notifier = new synthv1_sched_notifier(this);
+
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i)
 		m_params_ab[i] = synthv1_param::paramDefaultValue(synthv1::ParamIndex(i));
@@ -716,14 +720,10 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
-	if (pConfig)
-		m_ui.helpUseNativeDialogsAction->setChecked(pConfig->bUseNativeDialogs);
-
 	// Menu actions
-	QObject::connect(m_ui.helpUseNativeDialogsAction,
+	QObject::connect(m_ui.helpConfigureAction,
 		SIGNAL(triggered(bool)),
-		SLOT(helpUseNativeDialogs(bool)));
+		SLOT(helpConfigure()));
 	QObject::connect(m_ui.helpAboutAction,
 		SIGNAL(triggered(bool)),
 		SLOT(helpAbout()));
@@ -731,6 +731,18 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
+	// Special sample update notifications (eg. reverse)
+	QObject::connect(m_sched_notifier,
+		SIGNAL(notify(int)),
+		SLOT(updateSchedNotify(int)));
+
+	// General knob/dial  behavior init...
+	synthv1_config *pConfig = synthv1_config::getInstance();
+	if (pConfig) {
+		synthv1widget_dial::setDialMode(
+			synthv1widget_dial::DialMode(pConfig->iKnobDialMode));
+	}
+
 	// Epilog.
 	// QWidget::adjustSize();
 
@@ -776,21 +788,18 @@ void synthv1widget::setParamValue (
 
 float synthv1widget::paramValue ( synthv1::ParamIndex index ) const
 {
-	float fParamValue = 0.0f;
+	float fValue = 0.0f;
 
 	synthv1widget_knob *pKnob = paramKnob(index);
 	if (pKnob) {
-		fParamValue = pKnob->value();
+		fValue = pKnob->value();
 	} else {
-		synthv1 *pSynth = instance();
-		if (pSynth) {
-			const float *pParamPort = pSynth->paramPort(index);
-			if (pParamPort)
-				fParamValue = *pParamPort;
-		}
+		synthv1_ui *pSynthUi = ui_instance();
+		if (pSynthUi)
+			fValue = pSynthUi->paramValue(index);
 	}
 
-	return fParamValue;
+	return fValue;
 }
 
 
@@ -851,11 +860,11 @@ void synthv1widget::updateParamEx ( synthv1::ParamIndex index, float fValue )
 // Reset all param knobs to default values.
 void synthv1widget::resetParams (void)
 {
-	synthv1 *pSynth = instance();
-	if (pSynth == NULL)
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi == NULL)
 		return;
 
-	pSynth->reset();
+	pSynthUi->reset();
 
 	resetSwapParams();
 
@@ -918,15 +927,13 @@ void synthv1widget::updateParamValues (void)
 {
 	resetSwapParams();
 
-	synthv1 *pSynth = instance();
+	synthv1_ui *pSynthUi = ui_instance();
 
 	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
 		synthv1::ParamIndex index = synthv1::ParamIndex(i);
-		float fValue = synthv1_param::paramDefaultValue(index);
-		const float *pfParamPort
-			= (pSynth ? pSynth->paramPort(index) : NULL);
-		if (pfParamPort)
-			fValue = *pfParamPort;
+		const float fValue = (pSynthUi
+			? pSynthUi->paramValue(index)
+			: synthv1_param::paramDefaultValue(index));
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
@@ -997,23 +1004,14 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 	qDebug("synthv1widget::loadPreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	synthv1 *pSynth = instance();
-	if (pSynth == NULL)
-		return;
-
 	resetParamKnobs();
 	resetParamValues();
 
-	synthv1_param::loadPreset(pSynth, sFilename);
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi)
+		synthv1_param::loadPreset(pSynthUi->instance(), sFilename);
 
-	updateParamValues();
-
-	const QString& sPreset
-		= QFileInfo(sFilename).completeBaseName();
-
-	m_ui.Preset->setPreset(sPreset);
-	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
-	updateDirtyPreset(false);
+	updateLoadPreset(QFileInfo(sFilename).completeBaseName());
 }
 
 
@@ -1023,7 +1021,9 @@ void synthv1widget::savePreset ( const QString& sFilename )
 	qDebug("synthv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	synthv1_param::savePreset(instance(), sFilename);
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi)
+		synthv1_param::savePreset(pSynthUi->instance(), sFilename);
 
 	const QString& sPreset
 		= QFileInfo(sFilename).completeBaseName();
@@ -1040,14 +1040,54 @@ bool synthv1widget::queryClose (void)
 }
 
 
-// Menu actions.
-void synthv1widget::helpUseNativeDialogs ( bool bOn )
+// Preset status updater.
+void synthv1widget::updateLoadPreset ( const QString& sPreset )
 {
-	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
-	if (pConfig) {
-		pConfig->bUseNativeDialogs = bOn;
-		pConfig->bDontUseNativeDialogs = !pConfig->bUseNativeDialogs;
+	updateParamValues();
+
+	m_ui.Preset->setPreset(sPreset);
+	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
+	updateDirtyPreset(false);
+}
+
+
+// Notification updater.
+void synthv1widget::updateSchedNotify ( int stype )
+{
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi == NULL)
+		return;
+
+#ifdef CONFIG_DEBUG_0
+	qDebug("samplv1widget::updateSchedNotify(%d)", stype);
+#endif
+
+	switch (synthv1_sched::Type(stype)) {
+	case synthv1_sched::Programs: {
+		synthv1_programs *pPrograms = pSynthUi->programs();
+		synthv1_programs::Prog *pProg = pPrograms->current_prog();
+		if (pProg) updateLoadPreset(pProg->name());
+		break;
 	}
+	case synthv1_sched::Wave:
+	default:
+		break;
+	}
+}
+
+
+// Menu actions.
+void synthv1widget::helpConfigure (void)
+{
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi == NULL)
+		return;
+
+	synthv1widget_config form(this);
+
+	// Set programs database...
+	form.setPrograms(pSynthUi->programs());
+	form.exec();
 }
 
 
@@ -1122,15 +1162,12 @@ void synthv1widget::bpmSyncChanged (void)
 		return;
 
 	++m_iUpdate;
-	synthv1 *pSynth = instance();
-	if (pSynth) {
-		float *pBpmSync = pSynth->paramPort(synthv1::DEL1_BPMSYNC);
-		if (pBpmSync) {
-			const bool bBpmSync0 = (*pBpmSync > 0.0f);
-			const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
-			if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
-				*pBpmSync = (bBpmSync1 ? 1.0f : 0.0f);
-		}
+	synthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi) {
+		const bool bBpmSync0 = (pSynthUi->paramValue(synthv1::DEL1_BPMSYNC) > 0.0f);
+		const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
+		if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
+			pSynthUi->setParamValue(synthv1::DEL1_BPMSYNC, (bBpmSync1 ? 1.0f : 0.0f));
 	}
 	--m_iUpdate;
 }
